-- Copyright 2008 Steven Barth <steven@midlink.org>
-- Copyright 2011 Jo-Philipp Wich <jow@openwrt.org>
-- Licensed to the public under the Apache License 2.0.

local fs = require "nixio.fs"

config_xnet_dlms_master_client = Map("xnet_dlms_master_client", translate("xNet DLMS Master/ Client"),
	translate("Configure DLMS Master/ Client settings"))

-- Global section to enable/ disable config value
-------------------------------------------------
section_global = config_xnet_dlms_master_client:section(TypedSection, "global", translate("Enable/ Disable"))
section_global.anonymous = false
section_global.addremove = false
section_global.name = "global"

option_enabled = section_global:option(Flag, "enabled", translate("Enabled"),
	translate("Enable DLMS Master/ Client"))
option_enabled.rmempty=false

-- Polling Section
----------------------
section_polling = config_xnet_dlms_master_client:section(TypedSection, "polling", translate("DLMS Polling Configuration"))
section_polling.anonymous = true
section_polling.addremove = true
section_polling.template = "cbi/tblsection"

--id
polling_id = section_polling:option(Value, "id", translate("Name of this polling config"),
	translate("Choose a name for this configuration. Cannot contain space or special characters except _."))

logical_name = section_polling:option(Value, "logical_name", translate("Logical Name"),
	translate("Enter logical name of the DLMS profile in dot notation e.g. 1.0.254.1.240"))

read_mode = section_polling:option(ListValue, "read_mode", translate("Read Mode"),
	translate("DLMS Profiles can be read by Entry number in the table or by time duration range. Load profile/ Historical data is usually read by duration range, instantaneous profile are usually read by entry number"))
read_mode.default = "by_value"
read_mode:value("by_value", translate("By Value/ Entry counter"))
read_mode:value("by_range", translate("By Range/ Duration"))

polling_frequency = section_polling:option(Value, "polling_frequency", translate("Polling frequency"),
	translate("Enter value in seconds. Enter 0 to read once upon connection to the meter"))
polling_frequency.datatype="uinteger"


-- Connection Section
----------------------
section_connection = config_xnet_dlms_master_client:section(TypedSection, "connection", translate("DLMS Connections"))
section_connection.anonymous = true
section_connection.addremove = true

--id
connection_id = section_connection:option(Value, "id", translate("Name of the connection"),
	translate("Choose a name for this connection. Cannot contain space or special characters except _."))

--connection
connection = section_connection:option(ListValue, "connection", translate("Connection Type"),
	translate("Select Serial port or DLSM over TCP"))
connection.default = "serial"
connection:value("serial", translate("Serial Port"))
connection:value("tcp", translate("TCP/IP"))

--connection
serial_port = section_connection:option(ListValue, "serial_port", translate("Serial Port"))
serial_port.default = "/dev/ttyXR0"
serial_port:value("/dev/ttyS0", translate("COM1"))
serial_port:value("/dev/ttyXR0", translate("COM2"))
serial_port:value("/dev/ttyXR1", translate("COM3"))
serial_port:value("/dev/ttyXR2", translate("COM4"))
serial_port:value("/dev/ttyXR3", translate("COM5"))
serial_port:value("/dev/ttyUSB0", translate("USB1"))
serial_port:depends("connection", "serial")

--baud
baud = section_connection:option(ListValue, "baud", translate("Baud rate"))
baud.default = "9600"
baud:value("300", translate("300"))
baud:value("600", translate("600"))
baud:value("1200", translate("1200"))
baud:value("2400", translate("2400"))
baud:value("4800", translate("4800"))
baud:value("9600", translate("9600"))
baud:value("19200", translate("19200"))
baud:value("38400", translate("38400"))
baud:value("19200", translate("19200"))
baud:value("57600", translate("57600"))
baud:value("115200", translate("115200"))
baud:depends("connection", "serial")

--data_bit
data_bit = section_connection:option(ListValue, "data_bit", translate("Data Bits"))
data_bit.default = "8"
data_bit:value("7", translate("7 bits"))
data_bit:value("8", translate("8 bits"))
data_bit:value("9", translate("9 bits"))
data_bit:depends("connection", "serial")

--stop bits
parity = section_connection:option(ListValue, "parity", translate("Parity"))
parity.default = "N"
parity:value("N", translate("None"))
parity:value("E", translate("Even"))
parity:value("O", translate("Odd"))
parity:depends("connection", "serial")

--stop bits
stop_bit = section_connection:option(ListValue, "stop_bit", translate("Parity"))
stop_bit.default = "1"
stop_bit:value("0", translate("No stop bits"))
stop_bit:value("1", translate("1 stop bit"))
stop_bit:value("2", translate("2 stop bits"))
stop_bit:depends("connection", "serial")

--host
host = section_connection:option(Value, "host", translate("Host IP Address"), translate("IP Address of DLMS Meter"))
host.datatype = "ip4addr"
host.default = "192.168.1.254"
host:depends("connection", "tcp")

--port
port = section_connection:option(Value, "port", translate("Port"))
port.default = "4059"
port:depends("connection", "tcp")

--IEC
iec = section_connection:option(Flag, "iec", translate("DLMS/ IEC"),
	translate("Is this connection uses DLMS or IEC standard."))
iec.default = "0"
iec.rmempty=false

--HDLC Wrapper
hdlc_wrapper = section_connection:option(Flag, "hdlc_wrapper", translate("HDLC Wrapper"),
	translate("Is this TCP connection wrapper over HDLC. Usually used with Serial to TCP converters."))
hdlc_wrapper.default = "0"
hdlc_wrapper.rmempty=false
hdlc_wrapper:depends("connection", "tcp")


-- MQTT Section
----------------------
section_mqtt = config_xnet_dlms_master_client:section(TypedSection, "mqtt", translate("DLMS to MQTT IoT Configuration"))
section_mqtt.anonymous = true
section_mqtt.addremove = false

mqtt_host = section_mqtt:option(Value, "host", translate("MQTT Server"), translate("MQTT Server IP address/ host name"))
mqtt_host.datatype = "host"
mqtt_host.default = "192.168.1.254"

mqtt_port = section_mqtt:option(Value, "port", translate("Port"))
mqtt_port.default = "1883"
mqtt_port.datatype = "port"

mqtt_username = section_mqtt:option(Value, "username", translate("Username"))

mqtt_password = section_mqtt:option(Value, "password", translate("Password"))
mqtt_password.password = true

mqtt_topic = section_mqtt:option(Value, "topic", translate("Topic"), translate("Topic to send DLMS Data"))

clean_session = section_mqtt:option(Flag, "clean_session", translate("Clean Session"),
	translate("Clean session on reconnect by discaring any messages."))
clean_session.default = "0"
clean_session.rmempty=false

client_id = section_mqtt:option(Value, "client_id", translate("Client Id"))

will_topic = section_mqtt:option(Value, "will_topic", translate("Will Topic"))

will_payload = section_mqtt:option(Value, "will_payload", translate("Will Payload"))

will_qos = section_mqtt:option(Value, "will_qos", translate("Will QoC"))
will_qos.datatype="uinteger"
will_qos.max=3

will_retain = section_mqtt:option(Flag, "will_retain", translate("Will Retain"))
will_retain.default = "0"
will_retain.rmempty=false

reconnect_delay = section_mqtt:option(Value, "reconnect_delay", translate("Reconnection Delay"))
reconnect_delay.datatype="uinteger"

reconnect_exponential_backoff = section_mqtt:option(Flag, "reconnect_exponential_backoff", translate("Reconnect Exponential Backoff"))
reconnect_exponential_backoff.default = "0"
reconnect_exponential_backoff.rmempty=false

reconnect_delay_max = section_mqtt:option(Value, "reconnect_delay_max", translate("Maximum Reconnection Delay"))
reconnect_delay_max.datatype="uinteger"
reconnect_delay_max:depends("reconnect_exponential_backoff", "1")

max_inflight_messages = section_mqtt:option(Value, "max_inflight_messages", translate("Maximum In-flight messages"))
max_inflight_messages.datatype="uinteger"

return config_xnet_dlms_master_client
